@@ -12,7 +12,7 @@
  * software for any purpose.  It is provided "as is" without express or
  * implied warranty.
  *-----------------------------------------------------------------------------
- * $Id: tclXutil.c,v 5.4 1996/02/12 18:16:29 markd Exp $
+ * $Id: tclXutil.c,v 7.0 1996/06/16 05:31:00 markd Exp $
  *-----------------------------------------------------------------------------
  */
 
@@ -32,8 +32,11 @@ ReturnOverflow _ANSI_ARGS_((Tcl_Interp *interp));
 static int
 CallEvalErrorHandler _ANSI_ARGS_((Tcl_Interp  *interp));
 
-static Tcl_ChannelType *
-UnixFileChannelType _ANSI_ARGS_((Tcl_Interp *interp));
+static int
+ParseTranslationOption _ANSI_ARGS_((char *strValue));
+
+static char *
+FormatTranslationOption _ANSI_ARGS_((int value));
 
 /*
  * Used to return argument messages by most commands.
@@ -623,9 +626,7 @@ TclX_GetOpenChannel (interp, handle, accessMode)
     Tcl_Channel chan;
     int mode;
 
-    chan = Tcl_GetChannel (interp,
-                           TclSubstChannelName (handle),
-                           &mode);
+    chan = Tcl_GetChannel (interp, handle, &mode);
     if (chan == (Tcl_Channel) NULL) {
 	return NULL;
     }
@@ -696,9 +697,9 @@ TclX_GetOpenFnum (interp, handle, accessMode)
  *    Convert a channel to a file number.
  *
  * Parameters:
- *   o interp (I) - Current interpreter.
- *   o handle (I) - The file handle to convert.
- *   o direction (I) - TCL_READABLE or TCL_WRITABLE.
+ *   o channel (I) - Channel to get file number for.
+ *   o direction (I) - TCL_READABLE or TCL_WRITABLE, or zero.  If zero, then
+ *     return the first of the read and write numbers.
  * Returns:
  *   The file number or -1 if a file number is not associated with this access
  * direction.
@@ -711,10 +712,15 @@ TclX_ChannelFnum (channel, direction)
 {
     Tcl_File file;
 
-    file = Tcl_GetChannelFile (channel, direction);
-    if (file == NULL)
-        return -1;
-
+    if (direction == 0) {
+        file = Tcl_GetChannelFile (channel, TCL_READABLE);
+        if (file == NULL)
+            file = Tcl_GetChannelFile (channel, TCL_WRITABLE);
+    } else {
+        file = Tcl_GetChannelFile (channel, direction);
+        if (file == NULL)
+            return -1;
+    }
     return (int) Tcl_GetFileInfo (file, NULL);
 }
 
@@ -732,7 +738,7 @@ TclX_ChannelFnum (channel, direction)
  *   o mode (I) - Flags consisting of TCL_READABLE, TCL_WRITABLE.
  *   o isSocket (I) - TRUE if its a socket, FALSE otherwise.
  * Returns:
- *   The channel or NULL if an error occured.
+ *   The channel.  No error can occur.
  *-----------------------------------------------------------------------------
  */
 Tcl_Channel
@@ -740,36 +746,16 @@ TclX_SetupFileEntry (interp, fileNum, mode, isSocket)
     Tcl_Interp *interp;
     int         fileNum;
     int         mode;
-    int          isSocket;
+    int         isSocket;
 {
-    Tcl_File file;
     Tcl_Channel channel;
-    char *stdName;
 
-    switch (fileNum) {
-      case 0:
-        stdName = "stdin";
-        break;
-      case 1:
-        stdName = "stdout";
-        break;
-      case 2:
-        stdName = "stderr";
-        break;
-      default:
-        stdName = NULL;
-        break;
-    }
-
-    file = Tcl_GetFile ((ClientData) fileNum, TCL_UNIX_FD);
     if (isSocket) {
-        channel = Tcl_MakeTcpClientChannel (interp, file, stdName);
+        channel = Tcl_MakeTcpClientChannel ((ClientData) fileNum);
     } else {
-        channel = Tcl_MakeFileChannel (interp, file, mode, stdName);
-    }
-    if (channel == NULL) {
-        Tcl_FreeFile (file);
-        return NULL;
+        channel = Tcl_MakeFileChannel ((ClientData) fileNum,
+                                       (ClientData) fileNum,
+                                       mode);
     }
         
     Tcl_RegisterChannel (interp, channel);
@@ -807,59 +793,14 @@ Tcl_CloseForError (interp, channel, fileNum)
      Tcl_SetErrno (saveErrNo);
 }
 
-/*-----------------------------------------------------------------------------
- * Tcl_TicksToMS --
- *
- *   Convert clock ticks to milliseconds.
- *
- * Parameters:
- *   o numTicks (I) - Number of ticks.
- * Returns:
- *   Milliseconds.
- *-----------------------------------------------------------------------------
- */
-clock_t
-Tcl_TicksToMS (numTicks)
-     clock_t numTicks;
-{
-    static clock_t msPerTick = 0;
-
-    /*
-     * Some systems (SVR4) implement CLK_TCK as a call to sysconf, so lets only
-     * reference it once in the life of this process.
-     */
-    if (msPerTick == 0)
-        msPerTick = CLK_TCK;
-
-    if (msPerTick <= 100) {
-        /*
-         * On low resolution systems we can do this all with integer math. Note
-         * that the addition of half the clock hertz results in appoximate
-         * rounding instead of truncation.
-         */
-        return (numTicks) * (1000 + msPerTick / 2) / msPerTick;
-    } else {
-        /*
-         * On systems (Cray) where the question is ticks per millisecond, not
-         * milliseconds per tick, we need to use floating point arithmetic.
-         */
-        return ((numTicks) * 1000.0 / msPerTick);
-    }
-}
-
 /*-----------------------------------------------------------------------------
  * CallEvalErrorHandler --
  *
- *   Call the error handler specified in tclx_errorHandler.
+ *   Call the error handler function tclx_errorHandler, if it exists.  Passing
+ * it the result of the failed command.
  *
  * Parameters:
  *   o interp (I) - A pointer to the interpreter.
- *   o options (I) - Options controling the evaluation:
- *     o TCLX_EVAL_GLOBAL - Evaulate in the global context.
- *     o TCLX_EVAL_ERR_HANDLER - Call the user-specified error callback 
- *       specified in the global variable tclx_errorHandler if an error
- *       occurs.
- *   o cmd (I) - The command to evaluate.
  * Returns:
  *   The Tcl result code from the handler.  TCL_ERROR is returned and
  * result unchanged if not handler is available.
@@ -869,28 +810,32 @@ static int
 CallEvalErrorHandler (interp)
     Tcl_Interp  *interp;
 {
-    char         *errorHandler, *msgArg;
-    char         *msgArgv [2];
-    Tcl_DString   command;
-    int           result;
-
-    errorHandler = Tcl_GetVar (interp, "tclx_errorHandler", TCL_GLOBAL_ONLY);
-    if (errorHandler == NULL)
-        return TCL_ERROR;
-    
-    Tcl_DStringInit (&command);
-    Tcl_DStringAppend (&command, errorHandler, -1);
+    static char *ERROR_HANDLER = "tclx_errorHandler";
+    Tcl_CmdInfo cmdInfo;
+    Tcl_DString command;
+    int result;
 
     /*
-     * Use list merge to quote the argument, as it probably contains spaces.
+     * Check if the tclx_errorHandler function exists.  For backwards
+     * compatibility with TclX 7.4 we check to see if there is a variable
+     * by the same name holding the name of a procedrure.  Build up the command
+     * based on what we found.  The variable functionality is deprectated and
+     * should be removed eventually. ????
      */
-    msgArgv [0] = interp->result;
-    msgArgv [1] = NULL;
-    msgArg = Tcl_Merge (1, msgArgv);
+    if (!Tcl_GetCommandInfo (interp, ERROR_HANDLER, &cmdInfo)) {
+        char *errorHandler;
 
-    Tcl_DStringAppend (&command, " ", -1);
-    Tcl_DStringAppend (&command, msgArg, -1);
-    ckfree (msgArg);
+        errorHandler = Tcl_GetVar (interp, ERROR_HANDLER, TCL_GLOBAL_ONLY);
+        if (errorHandler == NULL)
+            return TCL_ERROR;  /* No handler specified */
+        Tcl_DStringInit (&command);
+        Tcl_DStringAppendElement (&command, errorHandler);
+    } else {
+        Tcl_DStringInit (&command);
+        Tcl_DStringAppendElement (&command, ERROR_HANDLER);
+    }
+
+    Tcl_DStringAppendElement (&command, interp->result);
 
     result = Tcl_GlobalEval (interp, Tcl_DStringValue (&command));
     if (result == TCL_ERROR) {
@@ -991,64 +936,6 @@ TclX_VarEval TCL_VARARGS_DEF(Tcl_Interp *, arg1)
     
     return result;
 }
-
-
-/*-----------------------------------------------------------------------------
- * TclX_Stdin --
- *
- *   Get a pointer to the stdin channel.  This is used so that I/O will be
- * redirected if the Tcl file table is changed.
- *
- * Parameters:
- *   o interp (I) - A pointer to the interpreter.
- * Returns:
- *   A pointer to the channel or NULL if its not open.
- *-----------------------------------------------------------------------------
- */
-Tcl_Channel
-TclX_Stdin (interp)
-    Tcl_Interp  *interp;
-{
-    return Tcl_GetChannel (interp, "stdin", NULL);
-}
-
-/*-----------------------------------------------------------------------------
- * TclX_Stdout --
- *
- *   Get a pointer to the stdout channel.  This is used so that I/O will be
- * redirected if the Tcl file table is changed.
- *
- * Parameters:
- *   o interp (I) - A pointer to the interpreter.
- * Returns:
- *   A pointer to the channel or NULL if its not open.
- *-----------------------------------------------------------------------------
- */
-Tcl_Channel
-TclX_Stdout (interp)
-    Tcl_Interp  *interp;
-{
-    return Tcl_GetChannel (interp, "stdout", NULL);
-}
-
-/*-----------------------------------------------------------------------------
- * TclX_Stderr --
- *
- *   Get a pointer to the stderr channel.  This is used so that I/O will be
- * redirected if the Tcl file table is changed.
- *
- * Parameters:
- *   o interp (I) - A pointer to the interpreter.
- * Returns:
- *   A pointer to the channel or NULL if its not open.
- *-----------------------------------------------------------------------------
- */
-Tcl_Channel
-TclX_Stderr (interp)
-    Tcl_Interp  *interp;
-{
-    return Tcl_GetChannel (interp, "stderr", NULL);
-}
 
 /*-----------------------------------------------------------------------------
  * TclX_WriteStr --
@@ -1069,3 +956,264 @@ TclX_WriteStr (channel, str)
 {
     return Tcl_Write (channel, str, strlen (str));
 }
+
+/*-----------------------------------------------------------------------------
+ * ParseTranslationOption --
+ *
+ *   Parse the string that represents the translation value for one channel
+ * direction.
+ *
+ * Parameters:
+ *   o strValue (I) - Channel translation value.
+ * Returns:
+ *   The integer option value.
+ *----------------------------------------------------------------------------- */
+static int
+ParseTranslationOption (strValue)
+    char *strValue;
+{
+    if (STREQU (strValue, "auto")) {
+        return TCLX_TRANSLATE_AUTO;
+    } else if (STREQU (strValue, "lf")) {
+        return TCLX_TRANSLATE_LF;
+    } else if (STREQU (strValue, "binary")) {
+        return TCLX_TRANSLATE_BINARY;
+    } else if (STREQU (strValue, "cr")) {
+        return TCLX_TRANSLATE_CR;
+    } else if (STREQU (strValue, "crlf")) {
+        return TCLX_TRANSLATE_CRLF;
+    } else if (STREQU (strValue, "platform")) {
+        return TCLX_TRANSLATE_PLATFORM;
+    }
+    panic ("ParseTranslationOption bug");
+}
+
+/*-----------------------------------------------------------------------------
+ * FormatTranslationOption --
+ *
+ *   Format the string that represents the translation value for one channel
+ * direction.
+ *
+ * Parameters:
+ *   o value (I) - Integer channel translation value.
+ * Returns:
+ *   The string option value.
+ *----------------------------------------------------------------------------
+ */
+static char *
+FormatTranslationOption (value)
+    int value;
+{
+    switch (value) {
+      case TCLX_TRANSLATE_AUTO:
+        return "auto";
+      case TCLX_TRANSLATE_LF:  /* Also binary */
+        return "lf";
+      case TCLX_TRANSLATE_CR:
+        return "cr";
+      case TCLX_TRANSLATE_CRLF:
+        return "crlf";
+      case TCLX_TRANSLATE_PLATFORM:
+        return "platform";
+      default:
+        panic ("FormatTranslationOption bug");
+    }
+}
+
+
+/*-----------------------------------------------------------------------------
+ * TclX_GetChannelOption --
+ *
+ *   C-friendly front end to Tcl_GetChannelOption.
+ *
+ * Parameters:
+ *   o channel (I) - Channel to get the option for.
+ *   o optionName (I) - One of the TCLX_COPT_* defines.
+ * Returns:
+ *   The integer option value define by TclX.
+ *-----------------------------------------------------------------------------
+ */
+int
+TclX_GetChannelOption (channel, option)
+    Tcl_Channel channel;
+    int         option;
+{
+    char *strOption;
+    Tcl_DString strValue;
+    int value;
+
+    Tcl_DStringInit (&strValue);
+
+    switch (option) {
+      case TCLX_COPT_BLOCKING:
+        strOption = "-blocking";
+        break;
+
+      case TCLX_COPT_BUFFERING:
+        strOption = "-buffering";
+        break;
+
+      case TCLX_COPT_TRANSLATION:
+        strOption = "-translation";
+        break;
+
+      default:
+        goto fatalError;
+    }
+
+    if (Tcl_GetChannelOption (channel, strOption, &strValue) != TCL_OK)
+        goto fatalError;
+
+    switch (option) {
+      case TCLX_COPT_BLOCKING:
+        if (strValue.string [0] == '0') {
+            value = TCLX_MODE_NONBLOCKING;
+        } else {
+            value = TCLX_MODE_BLOCKING;
+        }
+        break;
+
+      case TCLX_COPT_BUFFERING:
+        if (STREQU (strValue.string, "full")) {
+            value = TCLX_BUFFERING_FULL;
+        } else if (STREQU (strValue.string, "line")) {
+            value = TCLX_BUFFERING_LINE;
+        } else if (STREQU (strValue.string, "none")) {
+            value = TCLX_BUFFERING_NONE;
+        } else {
+            goto fatalError;
+        }
+        break;
+
+      case TCLX_COPT_TRANSLATION: {
+        /*
+         * The value returned is strange.  Its either a single word, or
+         * a list with a word for each file in the channel.  However, in
+         * Tcl 7.5, its actually retuned a list of a list, which is a bug.
+         * Handle this and code for working with a fixed version.  Hack
+         * the list rather than doing, since we know the possible values
+         * and this is much faster and easy to support both formats.
+         * ???Clean up once Tcl fixes the return.???
+         */
+        char *strValue1, *strValue2, *strScan;
+          
+        strValue1 = strValue.string;
+        if (strValue1 [0] == '{')
+            strValue1++;  /* Skip { if list of list */
+        strValue2 = strchr (strValue1, ' ');
+        if (strValue2 != NULL) {
+            strValue2 [0] = '\0';  /* Split into two strings. */
+            strValue2++;
+            strScan = strchr (strValue2, '}');
+            if (strScan != NULL)
+                *strScan = '\0';
+        } else {
+            strValue2 = strValue1;
+        }
+        value =
+          (ParseTranslationOption (strValue1) << TCLX_TRANSLATE_READ_SHIFT) |
+            ParseTranslationOption (strValue2);
+        break;
+      }
+    }
+    Tcl_DStringFree (&strValue);
+    return value;
+
+  fatalError:
+    panic ("TclX_GetChannelOption bug");
+}
+
+/*-----------------------------------------------------------------------------
+ * TclX_SetChannelOption --
+ *
+ *   C-friendly front end to Tcl_SetChannelOption.
+ *
+ * Parameters:
+ *   o interp (I) - Errors returned in result.
+ *   o channel (I) - Channel to set the option for.
+ *   o option (I) - One of the TCLX_COPT_* defines.
+ *   o value (I) - Value to set the option to (integer define).  Note, if
+ *     this is translation, it can either be the read and write directions
+ *     masked together or a single value.
+ * Result:
+ *   TCL_OK or TCL_ERROR;
+ *-----------------------------------------------------------------------------
+ */
+int
+TclX_SetChannelOption (interp, channel, option, value)
+    Tcl_Interp  *interp;
+    Tcl_Channel  channel;
+    int          option;
+    int          value;
+{
+    char *strOption, *strValue;
+    int readValue, writeValue;
+    char valueList [64];
+
+    switch (option) {
+      case TCLX_COPT_BLOCKING:
+        strOption = "-blocking";
+        switch (value) {
+          case TCLX_MODE_BLOCKING:
+            strValue = "1";
+            break;
+          case TCLX_MODE_NONBLOCKING:
+            strValue = "0";
+            break;
+          default:
+            goto fatalError;
+        }
+        break;
+
+      case TCLX_COPT_BUFFERING:
+        strOption = "-buffering";
+        switch (value) {
+          case TCLX_BUFFERING_FULL:
+            strValue = "full";
+            break;
+          case TCLX_BUFFERING_LINE:
+            strValue = "line";
+            break;
+          case TCLX_BUFFERING_NONE:
+            strValue = "none";
+            break;
+          default:
+            goto fatalError;
+        }
+        break;
+
+      case TCLX_COPT_TRANSLATION:
+        /*
+         * Hack a list together rather than allocate memory.  If values for
+         * read or write were not specified, specify both the same.
+         */
+        readValue = (value & TCLX_TRANSLATE_READ_MASK) >>
+            TCLX_TRANSLATE_READ_SHIFT;
+        writeValue = (value & TCLX_TRANSLATE_WRITE_MASK);
+        if (readValue == TCLX_TRANSLATE_UNSPECIFIED)
+            readValue = writeValue;
+        if (writeValue == TCLX_TRANSLATE_UNSPECIFIED)
+            writeValue = readValue;
+
+        strOption = "-translation";
+
+        valueList [0] = '\0';
+        valueList [sizeof (valueList) - 1] = '\0';  /* Overflow check */
+        strValue = valueList;
+
+        strcat (valueList, FormatTranslationOption (readValue));
+        strcat (valueList, " ");
+        strcat (valueList, FormatTranslationOption (writeValue));
+        if (valueList [sizeof (valueList) - 1] != '\0')
+            goto fatalError;
+        break;
+
+      default:
+        goto fatalError;
+    }
+
+    return Tcl_SetChannelOption (interp, channel, strOption, strValue);
+
+  fatalError:
+    panic ("TclX_SetChannelOption bug");
+}
